FrameHelper = {}
local defaultWindowHeight = 60
local defaultWindowWidth = 170

function FrameHelper:CreateFrame(titleText, height, width)
    height = height or defaultWindowHeight
    width = width or defaultWindowWidth

    local f = CreateFrame("Frame", titleText:gsub(" ", ""), UIParent)
    f:SetMovable(true)
    f:EnableMouse(true)
    f:SetUserPlaced(true)
    f:SetSize(width, height)
    f:SetPoint("CENTER")
    f:SetBackdrop({
        bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
        edgeFile = "Interface\\ChatFrame\\ChatFrameBackground",
        edgeSize = 1,
    })
    f:SetBackdropColor(0, 0, 0, .5)
    f:SetBackdropBorderColor(0, 0, 0)
    f:RegisterForDrag("LeftButton")
    f:SetScript("OnDragStart", f.StartMoving)
    f:SetScript("OnDragStop", f.StopMovingOrSizing)
    f:SetScript("OnHide", f.StopMovingOrSizing)

    f.closeButton = CreateFrame("Button", "CloseUI", f, "UIPanelCloseButton")
    f.closeButton:SetPoint("TOPRIGHT", f, "TOPRIGHT")
    f.closeButton:SetScript("OnClick", function()
        f:Hide()
    end)

    f.title = f:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
    f.title:SetPoint("TOPLEFT", 10, -10)
    f.title:SetText(titleText)

    f.buttons = {}
    return f
end

--[[
    Whenever we adjust the height we need to adjust the position as well
    We do this by:
        - Substracting the delta of the new height when we remove a button.
        - Or adding the delta of the new height when adding a button.
]]
function FrameHelper:UpdateHeight(f, newHeight)
    local point, relativeTo, relativePoint, xOfs, yOfs = f:GetPoint()
    local windowSettings = { 
        point = point,
        relativePoint = relativePoint,
        x = xOfs,
        y = yOfs
    }

    if windowSettings then
        local heightToAdjust = 0
        if f:GetHeight() < newHeight then -- Rows have been added
            heightToAdjust = (newHeight - f:GetHeight()) / 2
            windowSettings.y = windowSettings.y - heightToAdjust
        elseif f:GetHeight() > newHeight then -- Rows have been removed
            heightToAdjust = (f:GetHeight() - newHeight) / 2
            windowSettings.y = windowSettings.y + heightToAdjust
        end

        -- It is very important to clear all the points first, if we don't then the UI will move all over the screen, I have no idea why.
        f:ClearAllPoints()
        f:SetHeight(newHeight)
        f:SetPoint(windowSettings.point, nil, windowSettings.relativePoint, windowSettings.x, windowSettings.y)
    end
end

function FrameHelper:CreateClickableButton(f, text, onPreClick, onRemoveClick)
    local totalButtonHeight = (getn(f.buttons) * 20)

    local b = CreateFrame("Button", "MyButton", f, "UIPanelButtonTemplate,SecureActionButtonTemplate")
    b:SetSize(130,20)
    b:SetText(text)
    b:SetPoint("TOPLEFT", f, "TOPLEFT", 10, -30 - totalButtonHeight)
    b:SetScript("PreClick", onPreClick)

    local removeB = CreateFrame("Button", "MyButton", f, "UIPanelButtonTemplate,SecureActionButtonTemplate")
    removeB:SetSize(25,20)
    removeB:SetText("X")
    removeB.HiddenText = text
    removeB:SetPoint("TOPLEFT", f, "TOPLEFT", 140, -30 - totalButtonHeight)
    removeB:SetScript("OnClick", onRemoveClick)

    FrameHelper:UpdateHeight(f, defaultWindowHeight + totalButtonHeight)

    tinsert(f.buttons, { ["Button"] = b, ["RemoveButton"] = removeB })
end

function FrameHelper:RemoveClickAbleButton(line)
    if line then
        line["Button"]:Hide()
        line["Button"] = nil
        line["RemoveButton"]:Hide()
        line["RemoveButton"] = nil
    end
end

function FrameHelper:RemoveAllClickAbleButtons(f)
    for k,line in pairs(f.buttons) do 
        FrameHelper:RemoveClickAbleButton(line)
    end

    f.buttons = {}
end